@@ -104,3 +104,15 @@ class VolumePathNotRemoved(BrickException):
 
 class ProtocolNotSupported(BrickException):
     message = _("Connect to volume via protocol %(protocol)s not supported.")
+
+
+class TargetPortalNotFound(BrickException):
+    message = _("Unable to find target portal %(target_portal)s.")
+
+
+class FailedISCSITargetPortalLogin(BrickException):
+    message = _("Unable to login to iSCSI Target Portal")
+
+
+class BlockDeviceReadOnly(BrickException):
+    message = _("Block device %(device)s is Read-Only.")
